@@ -1,6 +1,7 @@
+from __future__ import absolute_import, division, print_function
 import numpy as np
 from .core import (
-    Operator, BlockOperator, BlockRowOperator, CompositionOperator)
+    BlockOperator, BlockRowOperator, CompositionOperator, Operator)
 from .flags import real
 from .utils import MPI
 
