@@ -50,6 +50,7 @@ __all__ = ['all_eq',
            'renumerate',
            'reshape_broadcast',
            'setting',
+           'settingerr',
            'split',
            'strelapsed',
            'strenum',
@@ -723,6 +724,18 @@ def setting(obj, attr, value):
         setattr(obj, attr, old_value)
 
 
+@contextmanager
+def settingerr(*args, **keywords):
+    """ Contextually set an error handling. """
+    old = np.seterr(*args, **keywords)
+    try:
+        yield
+    except:
+        raise
+    finally:
+        np.seterr(**old)
+
+
 def split(n, m, rank=None):
     """
     Return an iterator through the slices that partition a list of n elements
