@@ -4,11 +4,12 @@ import numpy as np
 from numpy.testing import assert_equal
 from pyoperators import Operator
 from pyoperators.utils import (
-    broadcast_shapes, cast, complex_dtype, first, float_dtype, ifirst,
-    first_is_not, ifirst_is_not, groupbykey, inspect_special_values,
-    interruptible, isscalarlike, izip_broadcast, least_greater_multiple, one,
-    pi, product, reshape_broadcast, setting, strenum, strplural, strshape,
-    uninterruptible, zero)
+    broadcast_shapes, cast, complex_dtype, first, first_is_not, float_dtype,
+    ifirst, ifirst_is_not, ilast, ilast_is_not, groupbykey,
+    inspect_special_values, interruptible, isscalarlike, izip_broadcast, last,
+    last_is_not, least_greater_multiple, one, pi, product, reshape_broadcast,
+    setting, settingerr, split, strenum, strplural, strshape, uninterruptible,
+    zero)
 from pyoperators.utils.testing import assert_eq, assert_raises, assert_same
 
 dtypes = [np.dtype(t) for t in (np.bool8, np.uint8, np.int8, np.uint16,
@@ -100,32 +101,32 @@ def test_float_dtype():
 
 def test_first1():
     assert first([1, 2, 3], lambda x: x > 1.5) == 2
-    assert first([1, 2, 3], lambda x: x > 1.5, reverse=True) == 3
+    assert last([1, 2, 3], lambda x: x > 1.5) == 3
 
 
 def test_first2():
     assert_raises(ValueError, first, [1, 2, 3], lambda x: x > 3)
-    assert_raises(ValueError, first, [1, 2, 3], lambda x: x > 3, reverse=True)
+    assert_raises(ValueError, last, [1, 2, 3], lambda x: x > 3)
 
 
 def test_ifirst1():
     assert ifirst([1, 2, 3], lambda x: x > 1.5) == 1
-    assert ifirst([1, 2, 3], lambda x: x > 1.5, reverse=True) == 2
+    assert ilast([1, 2, 3], lambda x: x > 1.5) == 2
 
 
 def test_ifirst2():
     assert_raises(ValueError, ifirst, [1, 2, 3], lambda x: x > 3)
-    assert_raises(ValueError, ifirst, [1, 2, 3], lambda x: x > 3, reverse=True)
+    assert_raises(ValueError, ilast, [1, 2, 3], lambda x: x > 3)
 
 
 def test_ifirst3():
     assert ifirst([1, 2, 2, 3], 2.) == 1
-    assert ifirst([1, 2, 2, 3], 2., reverse=True) == 2
+    assert ilast([1, 2, 2, 3], 2.) == 2
 
 
 def test_ifirst4():
     assert_raises(ValueError, ifirst, [1, 2, 3], 4)
-    assert_raises(ValueError, ifirst, [1, 2, 3], 4, reverse=True)
+    assert_raises(ValueError, ilast, [1, 2, 3], 4)
 
 
 def test_first_is_not():
@@ -134,17 +135,24 @@ def test_first_is_not():
     assert first_is_not([], None) is None
     assert first_is_not([None, None], None) is None
 
-    assert first_is_not([1, 2], 2, reverse=True) == 1
-    assert first_is_not([{}, None, None], None, reverse=True) == {}
-    assert first_is_not([], None, reverse=True) is None
-    assert first_is_not([None, None], None, reverse=True) is None
+    assert last_is_not([1, 2], 2) == 1
+    assert last_is_not([{}, None, None], None) == {}
+    assert last_is_not([], None) is None
+    assert last_is_not([None, None], None) is None
 
 
 def test_ifirst_is_not():
-    assert ifirst_is_not([1, 2], 2, reverse=True) == 0
-    assert ifirst_is_not([{}, None, None], None, reverse=True) == 0
-    assert_raises(ValueError, ifirst_is_not, [], None, reverse=True)
-    assert_raises(ValueError, ifirst_is_not, [None, None], None, reverse=True)
+    assert ifirst_is_not([1, 2, 2], 2) == 0
+    assert ifirst_is_not([2, 1, 1], 2) == 1
+    assert ifirst_is_not([{}, None, None], None) == 0
+    assert_raises(ValueError, ifirst_is_not, [], None)
+    assert_raises(ValueError, ifirst_is_not, [None, None], None,)
+
+    assert ilast_is_not([1, 2, 2], 2) == 0
+    assert ilast_is_not([2, 1, 1], 2) == 2
+    assert ilast_is_not([{}, None, None], None) == 0
+    assert_raises(ValueError, ilast_is_not, [], None)
+    assert_raises(ValueError, ilast_is_not, [None, None], None,)
 
 
 def test_groupbykey():
@@ -343,6 +351,35 @@ def test_setting():
     assert not hasattr(obj, 'otherattr')
 
 
+def test_settingerr():
+    ref1 = np.seterr()
+    ref2 = {'divide': 'ignore', 'invalid': 'ignore', 'over': 'ignore',
+            'under': 'ignore'}
+    ref3 = {'divide': 'raise', 'invalid': 'ignore', 'over': 'warn',
+            'under': 'ignore'}
+
+    with settingerr(all='ignore'):
+        assert_eq(np.seterr(), ref2)
+        with settingerr(divide='raise', over='warn'):
+            assert_eq(np.seterr(), ref3)
+        assert_eq(np.seterr(), ref2)
+    assert_eq(np.seterr(), ref1)
+
+
+def test_split():
+    def func(n, m):
+        slices = split(n, m)
+        assert_eq(len(slices), m)
+        x = np.zeros(n, int)
+        for s in slices:
+            x[s] += 1
+        assert_same(x, 1, broadcasting=True)
+        assert_eq([split(n, m, i) for i in range(m)], slices)
+    for n in range(4):
+        for m in range(1, 6):
+            yield func, n, m
+
+
 def test_strenum():
     assert_eq(strenum(['blue', 'red', 'yellow'], 'or'),
               "'blue', 'red' or 'yellow'")
